#
#
# Install script for BRU X11 Interface Kit
#
# Second stage
#
# Copyright (c) 1997, Enhanced Software Technologies, Inc.
# All rights Reserved
#
##########################################################################

# first, check tcl version
if {$tcl_version < 7.6} {
    puts "********************************************************************************"
    puts stderr "BRU's X11 Interface requires Tcl/Tk version 7.6/4.2 or better."
    puts stderr "Version found: $tcl_version"
    puts stderr ""
    puts stderr "Please visit www.scriptics.com to download a newer version of Tcl/Tk, "
    puts stderr "or set the environment variable TCLSH to point to the newer version"
    puts stderr "and rerun the installX script."
    exit 1
}

set tclshpath $env(TCLSH)
set archivepath [pwd]

proc FindInPath { name } {
    global env
    foreach dir [split $env(PATH) ":"] {
	if [ file executable "$dir/$name" ] {
	    return "$dir/$name"
	}
    }
    # if here, then not found
    return ""
}

# check for bru installation
if {[catch {exec bru -h 2>/dev/null >/dev/null}]} {
    puts stderr "ABORTING: BRU not properly installed"
    exit 1
}

# make sure have wish
set wishpath ""
while {$wishpath == ""} {
    foreach wish [list wish8.2 wish8.1 wish8.0 wish4.2 wish] {
	set wishpath [FindInPath $wish]
	if {$wishpath != ""} {
	    break
	}
    }

    # not found, ask user
    if {$wishpath == ""} {
	puts "********************************************************************************"
	puts "Unable to find wish executable in current path.  Please enter the path to the"
	puts "wish version 4.2 or greater executable.  You may need to download and compile"
	puts "a current version of Tcl/Tk from www.scriptics.com.  Enter 'q' to abort."
	puts -nonewline "Path: "
	flush stdout
	set wishpath [gets stdin]
	if {[eof stdin] || $wishpath == "q" || $wishpath == "Q"} {
	    exit 1
	}
	puts ""
    }

    if {! [ file executable $wishpath ] } {
	puts "$wishpath does not exist!"
	puts ""
	set wishpath ""
    }
}

# set installation paths
set libpath ""
while {$libpath == ""} {
    puts "********************************************************************************"
    puts "Please enter the location where you would like the miscellaneous files for BRU's"
    puts "X11 Interface to be installed.  The installation script will create an"
    puts "'xbru' subdirectory of the given path."
    puts -nonewline "Path \[/usr/local\]: "
    flush stdout

    set libpath [gets stdin]
    if {[eof stdin]} {
	exit 1
    }
    puts ""

    if {$libpath == ""} {
	set libpath "/usr/local"
    }

    # bad things will happen if we try to install into an xbru/xbru directory, and
    # the parent xbru directory contains an existing installation
    set parts [file split $libpath]
    while {[lindex $parts end] == "xbru"} {
	set parts [lreplace $parts end end]
    }
    set libpath [eval "file join $parts"]

    if {![file isdirectory $libpath]} {
	if {![file exists $libpath]} {
	    puts -nonewline "$libpath doesn't exist, would you like to create it? (Y/N) \[Y\]: "
	    flush stdout
	    
	    set ans [gets stdin]
	    if {[eof stdin]} { exit 1 }
	    puts ""
	    if {$ans == ""} {
		set ans "Y"
	    }
	    if {$ans != "Y" && $ans != "y"} {
		set libpath ""
	    } else {
		# diretory will be created later
	    }
	} else {
	    puts stderr "$libpath exists, but is not a directory!"
	    set libpath ""
	}
    }
}

set binpath ""
while {$binpath == ""} {
    puts "********************************************************************************"
    puts "Please enter the location where you would like the executable files for BRU's"
    puts "X11 Interface to be installed.  This directory should be included in"
    puts "the system path."
    puts "Current path: $env(PATH)"
    puts -nonewline "Path \[/usr/local/bin\]: "
    flush stdout

    set binpath [gets stdin]
    if {[eof stdin]} {
	exit 1
    }
    puts ""

    if {$binpath == ""} {
	set binpath "/usr/local/bin"
    }

    if {![file isdirectory $binpath]} {
	if {![file exists $binpath]} {
	    puts -nonewline "$binpath doesn't exist, would you like to create it? (Y/N) \[Y\]: "
	    flush stdout
	    
	    set ans [gets stdin]
	    if {[eof stdin]} { exit 1 }
	    puts ""
	    if {$ans == ""} {
		set ans "Y"
	    }
	    if {$ans != "Y" && $ans != "y"} {
		set binpath ""
	    } else {
		# creating directory
	    }
	} else {
	    puts stderr "$binpath exists, but is not a directory!"
	    set binpath ""
	}
    }
}

# confirm settings
puts "********************************************************************************"
puts "Confirm Installation Settings:"
puts ""
puts "  tclsh location:      $tclshpath"
puts "  wish location:       $wishpath"
puts "  miscellaneous files: $libpath/xbru"
puts "  executable files:    $binpath"
puts ""
puts -nonewline "Continue with installation? (Y/N) \[Y\]: "
flush stdout

set ans [gets stdin]
if {[eof stdin]} {
    exit 1
}

if {$ans == ""} {
    set ans "Y"
}

if {$ans != "Y" && $ans != "y"} {
    puts "You can restart the installation by running the installX script"
    exit 1
}

# proceed with install

# Attempt to find old xbru installs
set oldxbru [FindInPath xbru]
set oldbindir ""
set oldlibdir ""

if {$oldxbru == ""} {
    puts "No xbru in \$PATH: assuming fresh install..."
} else {
    if {[catch {
	# there are 2 ways to find the old lib directory.  First, with 1.6 or before,
	# xbru is a link to the xbru.tcl file in the lib directory.  Of course, our
	# found xbru might be a link to another xbru, which is the link to the lib dir,
	# so we have to allow for that.

	# With 1.7 or later, xbru is a script file that defines the XBRUDIR environment
	# variable to tell us where the libdir is. Again, our found xbru could be a link
	# to this script file, so we have to be prepared for that...

	# follow the yellow-brick road...
	set tmp $oldxbru
	while {[file type $tmp] == "link"} {
	    # this handles relative as well as absolute links.
	    set tmp [file join [file dirname $tmp] [file readlink $tmp]]
	}

	# tmp should now contain the destination for the xbru links, if any.
	if {[file extension $tmp] == ".tcl"} {
	    # 1.6 or earlier
	    set oldlibdir [file dirname $tmp]
	} else {
	    # look for XBRUDIR in xbru script
	    set fp [open $tmp r]
	    set commands [split [read $fp] "\n;"]
	    close $fp
	    foreach command $commands {
		set command [string trim $command]
		if {[string first "XBRUDIR" $command] == 0} {
		    # found it!
		    set oldlibdir [string trim [lindex [split $command "="] 1]]
		    break
		}
	    }
	    if {$oldlibdir == ""} {
		error "PANIC: no XBRUDIR set in $tmp"
	    }
	}
    } err]} {
	# error occured above
	set oldxbru ""
	set oldbindir ""
	set oldlibdir ""

	puts stderr ""
	puts stderr $err
	
	puts stderr "Skipping attempt to merge old installation.  May need to salvage"
	puts stderr "saved definitions and schedules manually!"
	puts stderr ""
	puts -nonewline "Press <Enter> to continue: "
	flush stdout
	gets stdin
    } else {
	set oldbindir [file dirname $oldxbru]
	puts "Old xbru found in: $oldbindir"
	puts "Old lib directory is: $oldlibdir"
    }
}

# Now, if we found an old installation, and there were no other problems,
# we have oldbindir and oldlibdir set to the old location.  This can now
# be taken into account below to preserve saved definitions and schedules.

if {[file exists "$libpath/xbru"] && ![file isdirectory "$libpath/xbru"]} {
    puts stderr "ABORTING: $libpath/xbru exists but is not a directory"
    exit 1
}

if {![file isdirectory "$libpath/xbru"]} {
    puts "creating $libpath/xbru..."
    file mkdir "$libpath/xbru"
}

if {[catch {cd "$libpath/xbru"}]} {
    puts stderr "ABORTING: failed to cd to $libpath/xbru"
    exit 1
}

set remove_script [open "$libpath/xbru/uninstall.sh" {WRONLY TRUNC CREAT}]
puts $remove_script "printf \"This script removes XBRU from your system.  Are you sure? \[y/N] \""
puts $remove_script "read ans"
puts $remove_script "if \[ \"\$ans\" != \"Y\" -a \"\$ans\" != \"y\" ] ; then"
puts $remove_script "  exit 0"
puts $remove_script "fi"
puts $remove_script ""
puts $remove_script "rm -rvf $libpath/xbru"

exec chmod 700 "$libpath/xbru/uninstall.sh"

set files [glob $archivepath/xbru*.bru]
set files [lsort $files]
set archive [lindex $files end]
puts "extracting files from $archive..."
if {[catch {exec bru -xBvf $archive -ua}]} {
    puts stderr "ABORTING: file extraction failed"
    exit 1
}

exec chmod 1777 "$libpath/xbru"

if {"$oldlibdir" != ""} {
    # preserve any saved definitions and schedules
    if {"$oldlibdir" == "$libpath/xbru"} {
	# same directory install -- no need.  Some other cleanup though
	puts "Will use existing definitions/schedules..."
	if {[file exists "$libpath/xbru/runschedule"]} {
	    # replaced by runschedule.tcl
	    puts stderr "Removing obsolete $libpath/xbru/runschedule..."
	    file delete "$libpath/xbru/runschedule"
	}
    } else {
	# copy old definitions/schedules to new location
	if {[file exists "$oldlibdir/bruschedules"]} {
	    puts "Copying old definitions/schedules..."
	    puts "  $oldlibdir/bruschedules -> $libpath/xbru/bruschedules"
	    file copy "$oldlibdir/bruschedules" "$libpath/xbru/bruschedules"
	}
	foreach path [glob "$oldlibdir/*"] {
	    if {[file isdirectory $path]} {
		puts "  $path -> $libpath/xbru/[file tail $path]"
		file copy $path "$libpath/xbru/[file tail $path]"
	    }
	}
    }
}

if {![file exists "$libpath/xbru/bruschedules"]} {
    # create initial bruschedules file
    puts "creating $libpath/xbru/bruschedules"
    set fp [open "$libpath/xbru/bruschedules" {WRONLY TRUNC CREAT}]
    close $fp
    exec chmod 0666 "$libpath/xbru/bruschedules"
}

if {![file isdirectory "$binpath"]} {
    file mkdir "$binpath"
}

if {[file exists "$binpath/xbru"]} {
    puts "removing old $binpath/xbru"
    file delete $binpath/xbru
}

puts "creating $binpath/xbru"
if {[catch {set fp [open "$binpath/xbru" {RDWR TRUNC CREAT}]}]} {
    puts stderr "ABORTING: failed to open $binpath/xbru"
    puts stderr $errorInfo
}

puts $fp "#!/bin/sh"
puts $fp "# BRU X11 Interface"
puts $fp "# Copyright 1998 by Enhanced Software Technologies, Inc"
puts $fp "#"
puts $fp {if [ "$TCLSH" = "" ] ; then}
puts $fp "    TCLSH=$tclshpath ; export TCLSH"
puts $fp "fi"
puts $fp ""
puts $fp {if [ "$WISH" = "" ] ; then}
puts $fp "    WISH=$wishpath ; export WISH"
puts $fp "fi"
puts $fp ""
puts $fp {if [ "$XBRUDIR" = "" ] ; then}
puts $fp "    XBRUDIR=$libpath/xbru ; export XBRUDIR"
puts $fp "fi"
puts $fp ""
puts $fp {if [ "$1" = "" ] ; then}
puts $fp {    $WISH $XBRUDIR/xbru.tcl}
puts $fp {elif [ "$1" = "-config" ] ; then}
puts $fp {    $WISH $XBRUDIR/TabEdit.tcl}
puts $fp "else"
puts $fp {    echo "Usage: xbru [-config]"}
puts $fp "fi"
puts $fp ""
puts $fp "exit $?"

close $fp
exec chmod 0755 "$binpath/xbru"

puts $remove_script "rm -vf $binpath/xbru"

if {[file exists "$binpath/bruschedule"]} {
    puts "removing old $binpath/bruschedule"
    file delete $binpath/bruschedule
}

puts "creating $binpath/bruschedule"
if {[catch {set fp [open "$binpath/bruschedule" {RDWR TRUNC CREAT}]}]} {
    puts stderr "ABORTING: failed to open $binpath/bruschedule"
    puts stderr $errorInfo
}

puts $fp "#!/bin/sh"
puts $fp "# BRU X11 Interface"
puts $fp "# Copyright 1998 by Enhanced Software Technologies, Inc"
puts $fp "#"
puts $fp {if [ "$TCLSH" = "" ] ; then}
puts $fp "    TCLSH=$tclshpath ; export TCLSH"
puts $fp "fi"
puts $fp ""
puts $fp {if [ "$WISH" = "" ] ; then}
puts $fp "    WISH=$wishpath ; export WISH"
puts $fp "fi"
puts $fp ""
puts $fp {if [ "$XBRUDIR" = "" ] ; then}
puts $fp "    XBRUDIR=$libpath/xbru ; export XBRUDIR"
puts $fp "fi"
puts $fp ""
puts $fp "\$TCLSH \$XBRUDIR/runschedule.tcl"
puts $fp "exit $?"

close $fp
exec chmod 0755 "$binpath/bruschedule"

puts $remove_script "rm -vf $binpath/bruschedule"
close $remove_script

if {("$oldbindir" != "") && ("$oldbindir" != "$binpath")} {
    if {[file exists "$oldbindir/xbru"]} {
	puts "removing $oldbindir/xbru..."
	file delete "$oldbindir/xbru"
    }
    if {[file exists "$oldbindir/bruconfig"]} {
	puts "removing obsolete $oldbindir/bruconfig..."
	file delete "$oldbindir/bruconfig"
    }
    if {[file exists "$oldbindir/bruschedule"]} {
	puts "removing $oldbindir/bruschedule..."
	file delete "$oldbindir/bruschedule"
    }
}

# remove old libpath if exists
if {("$oldlibdir" != "") && ("$oldlibdir" != "$libpath/xbru")} {
    puts "removing old installation at $oldlibdir"
    file delete -force $oldlibdir
}

puts ""
puts "Done!"
puts ""

if { $tcl_platform(os) == "Linux" } {
    if {![catch {exec grep "bfmcmd" /etc/brutab}]} {
	puts "********************************* IMPORTANT ************************************"
	puts "For Linux systems, we recommend running XBRU *without* setting bfmcmd.  You     "
	puts "may wish to remove settings for bfmcmd from your /etc/brutab file.              "
	puts "********************************* IMPORTANT ************************************"
    }
}

puts "********************************************************************************"
puts "Be sure to configure cron to run $binpath/bruschedule every 5 minutes to"
puts "activate the BRU X11 Interface scheduler.  See the BRU X11 User's"
puts "Guide and/or 'man crontab' for additional details."
puts "********************************************************************************"
